% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "Atomic Norm Minimization for Modal Analysis from Random and Compressed Samples"
% IEEE transactions on signal processing, 2018
% by Shuang Li, Dehui Yang, Gongguo Tang, and Michael B. Wakin
% This code is used to reproduce Figure 5.

%%
clc; clear; close all;
Freq=[0.1 0.15 0.5];       % true frequencies(Hz)
K=size(Freq,2);            % number of active modes
Ts=1;                      % sampling interval
N=10;                      % number of sensors
A=diag([1 2 3]);           % amplitudes
M=80;                      % number of samples
T=0:Ts:(M-1)*Ts;
T=T';
Q=exp(1j*2*pi*T*Freq);

%% Orthogonal modal shapes
OMShape=randn(N,K); % True mode shapes
if N > 1
    OMShape = OMShape*diag(1./norms(OMShape));
else
    OMShape = ones(1,K);
end
OMShape=orth(OMShape);                    % orthogonal modal shapes
OData=transpose(Q*A*transpose(OMShape));  % data matrix

% ====================SVD===========================
[u,~,~] = svd(OData);   
OMShape_SVD=u(:,1:K);
OMAC_SVD=abs(OMShape'*OMShape_SVD);
idx1=find(OMAC_SVD(1,:)==max(OMAC_SVD(1,:)));
idx2=find(OMAC_SVD(2,:)==max(OMAC_SVD(2,:)));
idx3=find(OMAC_SVD(3,:)==max(OMAC_SVD(3,:)));
OMShape_SVD=OMShape_SVD(:,[idx1 idx2 idx3]);    % recovered mode shapes
OMAC_SVD=diag(abs(OMShape'*OMShape_SVD))'       % compute MAC

% ============Random Spatial Compression============
B=randn(N,M);
BData=zeros(M,1);      % compressed data
for ii=1:M
    BData(ii)=B(:,ii)'*OData(:,ii);
end

cvx_begin quiet
variable u0;
variable u(M-1) complex;
variable Tu(M,M) hermitian;
variable X(N,M) complex;
variable W(N,N) Hermitian;
dual variable Y0
minimize(0.5/M*trace(Tu)+0.5*trace(W))
subject to
Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
Y0:[Tu   X'; ...
    X   W] == semidefinite(M+N,M+N);
for ii=1:M
    BData(ii)==B(:,ii)'*X(:,ii);
end
cvx_end

Y0=-2*Y0(M+1:M+N,1:M);     
Y=zeros(N,M);               % dual solution
for i=1:M
    Y(:,i)=Y0(:,M-i+1);
end

f=0:0.01:1;
a=exp(1j*2*pi*T*f);

QQ=Y*a;                     % dual polynomial 
if N > 1
    NormQ=norms(QQ);
else
    NormQ=abs(QQ);
end

[pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
if size(locs,2) < K
    OFreq_Spa(1:size(locs,2))=f(locs);         % recovered frequencies
    OMShape_Spa(:,1:size(locs,2))=QQ(:,locs);  % recovered mode shapes
else
    OFreq_Spa=f(locs(1:K));                    % recovered frequencies
    OMShape_Spa=QQ(:,locs(1:K));               % recovered mode shapes
end


MAC_Spa = abs(OMShape'*OMShape_Spa);
idx1=find(MAC_Spa(1,:) == max(MAC_Spa(1,:)));
idx2=find(MAC_Spa(2,:) == max(MAC_Spa(2,:)));
idx3=find(MAC_Spa(3,:) == max(MAC_Spa(3,:)));
OMShape_Spa = OMShape_Spa(:,[idx1(1) idx2(1) idx3(1)]); 
OMAC_Spa = diag(abs(OMShape'*OMShape_Spa))'
OFreq_Spa


%% Correlated modal shapes
CMShape=randn(N,K); % True mode shapes
CMShape(:,2)=CMShape(:,1);
CMShape(1,2)=CMShape(1,2)+0.1;  % The first two mode shapes are very close

if N > 1
    CMShape = CMShape*diag(1./norms(CMShape));
else
    CMShape = ones(1,K);
end

CData=transpose(Q*A*transpose(CMShape));

% ====================SVD===========================
[u,s,~] = svd(CData);  % !!!!!!
CMShape_SVD=u(:,1:K);
CMAC_SVD=abs(CMShape'*CMShape_SVD);
idx1=find(CMAC_SVD(1,:)==max(CMAC_SVD(1,:)));
idx2=find(CMAC_SVD(2,:)==max(CMAC_SVD(2,:)));
idx3=find(CMAC_SVD(3,:)==max(CMAC_SVD(3,:)));
CMShape_SVD=CMShape_SVD(:,[idx1 idx2 idx3]);      % recovered mode shapes
CMAC_SVD=diag(abs(CMShape'*CMShape_SVD))'

% ============Random Spatial Compression============
B=randn(N,M);
BData=zeros(M,1);       % compressed data
for ii=1:M
    BData(ii)=B(:,ii)'*CData(:,ii);
end

cvx_begin quiet
variable u0;
variable u(M-1) complex;
variable Tu(M,M) hermitian;
variable X(N,M) complex;
variable W(N,N) Hermitian;
dual variable Y0
minimize(0.5/M*trace(Tu)+0.5*trace(W))
subject to
Tu == toeplitz([u0;conj(u)],[u0;conj(u)]');
Y0:[Tu   X'; ...
    X   W] == semidefinite(M+N,M+N);
for ii=1:M
    BData(ii)==B(:,ii)'*X(:,ii);
end
cvx_end

Y0=-2*Y0(M+1:M+N,1:M);
Y=zeros(N,M);              % dual solution
for i=1:M
    Y(:,i)=Y0(:,M-i+1);
end

f=0:0.01:20;
a=exp(1j*2*pi*T*f);
QQ=Y*a;                    % dual polynomial 
if N > 1
    NormQ=norms(QQ);
else
    NormQ=abs(QQ);
end


[pks,locs] = findpeaks(NormQ,'minpeakdistance',1,'MinPeakHeight',0.99);
if size(locs,2) < K
    CFreq_Spa(1:size(locs,2))=f(locs);          % estimated frequencies
    CMShape_Spa(:,1:size(locs,2))=QQ(:,locs);   % estimated mode shapes
else
    CFreq_Spa=f(locs(1:K));
    CMShape_Spa=QQ(:,locs(1:K));          
end


MAC_Spa = abs(CMShape'*CMShape_Spa);
idx1=find(MAC_Spa(1,:) == max(MAC_Spa(1,:)));
idx2=find(MAC_Spa(2,:) == max(MAC_Spa(2,:)));
idx3=find(MAC_Spa(3,:) == max(MAC_Spa(3,:)));
CMShape_Spa = CMShape_Spa(:,[idx1(1) idx2(1) idx3(1)]);      % recovered mode shapes
CMAC_Spa = diag(abs(CMShape'*CMShape_Spa))'


%% Show results
fs=29;
lw=6;
figure

subplot(3,1,1)
plot(abs(OMShape(:,1)),'r-','linewidth',lw);hold on;
plot(abs(OMShape_Spa(:,1)),'b--','linewidth',lw);hold on;
plot(abs(OMShape_SVD(:,1)),'g-.','linewidth',lw);hold on;
legend('True', 'Spatial', 'SVD')
axis tight;
ylabel('Mode Shape  1','FontSize',fs)
title('Orthogonal Mode Shapes','FontSize',fs)
set(gca,  'FontSize'  , fs )

subplot(3,1,2)
plot(abs(OMShape(:,2)),'r-','linewidth',lw);hold on;
plot(abs(OMShape_Spa(:,2)),'b--','linewidth',lw);hold on;
plot(abs(OMShape_SVD(:,2)),'g-.','linewidth',lw);hold on;
axis tight;
ylabel('Mode Shape  2','FontSize',fs)
set(gca,  'FontSize'  , fs )

subplot(3,1,3)
plot(abs(OMShape(:,3)),'r-','linewidth',lw);hold on;
plot(abs(OMShape_Spa(:,3)),'b--','linewidth',lw);hold on;
plot(abs(OMShape_SVD(:,3)),'g-.','linewidth',lw);hold on;
axis tight;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 3','FontSize',fs)
set(gca,  'FontSize'  , fs )



figure

subplot(3,1,1)
plot(abs(CMShape(:,1)),'r-','linewidth',lw);hold on;
plot(abs(CMShape_Spa(:,1)),'b--','linewidth',lw);hold on;
plot(abs(CMShape_SVD(:,1)),'g-.','linewidth',lw);hold on;
axis tight;
legend('True', 'Spatial', 'SVD')
title('Correlated Mode Shapes','FontSize',fs)
ylabel('Mode Shape 1','FontSize',fs)
set(gca,  'FontSize'  , fs )

subplot(3,1,2)
plot(abs(CMShape(:,2)),'r-','linewidth',lw);hold on;
plot(abs(CMShape_Spa(:,2)),'b--','linewidth',lw);hold on;
plot(abs(CMShape_SVD(:,2)),'g-.','linewidth',lw);hold on;
axis tight;
ylabel('Mode Shape 2','FontSize',fs)
set(gca,  'FontSize'  , fs )

subplot(3,1,3)
plot(abs(CMShape(:,3)),'r-','linewidth',lw);hold on;
plot(abs(CMShape_Spa(:,3)),'b--','linewidth',lw);hold on;
plot(abs(CMShape_SVD(:,3)),'g-.','linewidth',lw);hold on;
axis tight;
xlabel('Node','FontSize',fs)
ylabel('Mode Shape 3','FontSize',fs)
set(gca,  'FontSize'  , fs )



